// src/utils/rangeMappers/index.ts
var mapVisibleRange = (model, cb) => {
  const result = [];
  for (let from = model.from, to = model.to; from < to; from++) {
    result.push(cb(from));
  }
  return result;
};
var mapVisibleRangeWithOffset = (model, cb) => {
  const result = [];
  for (let from = model.from, to = model.to, delta = model.getOffset(from); from < to; delta += model.getSize(from), from++) {
    result.push(cb(from, delta));
  }
  return result;
};

// src/hooks/useSyncedStyles/index.ts
import { useState } from "react";

// src/hooks/useIsomorphicLayoutEffect/index.ts
import { useLayoutEffect, useEffect } from "react";
var useIsomorphicLayoutEffect_default = typeof window !== "undefined" ? useLayoutEffect : useEffect;

// src/hooks/useSyncedStyles/index.ts
import { VirtualScrollerEvent } from "@af-utils/virtual-core";
var SIZE_PROVIDER_STYLE_BASE = {
  position: "relative",
  overflow: "hidden",
  contain: "strict",
  // otherwise may go above sticky header
  zIndex: -1
};
var SIZE_PROVIDER_STYLE_HORIZONTAL = {
  ...SIZE_PROVIDER_STYLE_BASE,
  height: "100%"
};
var SIZE_PROVIDER_STYLE_VERTICAL = {
  ...SIZE_PROVIDER_STYLE_BASE,
  width: "100%"
};
var SCROLL_PROVIDER_STYLE_BASE = {
  position: "absolute",
  contain: "strict",
  overflow: "hidden",
  top: 0,
  left: 0
};
var SCROLL_PROVIDER_STYLE_HORIZONTAL = {
  ...SCROLL_PROVIDER_STYLE_BASE,
  display: "flex",
  height: "100%"
};
var SCROLL_PROVIDER_STYLE_VERTICAL = {
  ...SCROLL_PROVIDER_STYLE_BASE,
  display: "block",
  width: "100%"
};
var useSyncedStyles = (model) => {
  const [outer, outerRef] = useState(null);
  const [inner, innerRef] = useState(null);
  useIsomorphicLayoutEffect_default(() => {
    if (model && outer && inner) {
      const updateSizeStyle = () => {
        outer.style[model.horizontal ? "width" : "height"] = model.scrollSize + "px";
      };
      const updateScrollStyle = () => {
        const fromOffset = model.getOffset(model.from);
        const toOffset = model.getOffset(model.to);
        inner.style.transform = `translate${model.horizontal ? "X" : "Y"}(${fromOffset}px)`;
        inner.style[model.horizontal ? "width" : "height"] = toOffset - fromOffset + "px";
      };
      const unsubSize = model.on(updateSizeStyle, [
        VirtualScrollerEvent.SCROLL_SIZE
      ]);
      const unsubScroll = model.on(updateScrollStyle, [
        VirtualScrollerEvent.RANGE,
        VirtualScrollerEvent.SIZES
      ]);
      Object.assign(
        outer.style,
        model.horizontal ? SIZE_PROVIDER_STYLE_HORIZONTAL : SIZE_PROVIDER_STYLE_VERTICAL
      );
      Object.assign(
        inner.style,
        model.horizontal ? SCROLL_PROVIDER_STYLE_HORIZONTAL : SCROLL_PROVIDER_STYLE_VERTICAL
      );
      updateSizeStyle();
      updateScrollStyle();
      return () => {
        unsubSize();
        unsubScroll();
      };
    }
  }, [model, outer, inner]);
  return [outerRef, innerRef];
};
var useSyncedStyles_default = useSyncedStyles;

// src/hooks/useComponentSubscription/index.ts
import { useMemo } from "react";
import { VirtualScrollerEvent as VirtualScrollerEvent2 } from "@af-utils/virtual-core";
import { useSyncExternalStore } from "use-sync-external-store/shim/index.js";
function getSingleEventHash(e) {
  switch (e) {
    case VirtualScrollerEvent2.RANGE:
      return this.to ** 2 + this.from;
    case VirtualScrollerEvent2.SCROLL_SIZE:
      return this.scrollSize;
    default:
      return this.sizesHash;
  }
}
var useComponentSubscription = (model, events) => {
  const [subscribe, getHash] = useMemo(
    () => [
      (listener) => model.on(listener, events),
      () => events.map(getSingleEventHash, model).join()
    ],
    [model, events]
  );
  useSyncExternalStore(subscribe, getHash, getHash);
};
var useComponentSubscription_default = useComponentSubscription;

// src/components/Subscription/index.tsx
var Subscription = (props) => {
  useComponentSubscription_default(props.model, props.events);
  return props.children();
};
var Subscription_default = Subscription;

// src/components/List/index.tsx
import { VirtualScrollerEvent as VirtualScrollerEvent3 } from "@af-utils/virtual-core";
import { useCallback } from "react";
import { jsx, jsxs } from "react/jsx-runtime";
var List = (props) => {
  const {
    model,
    children: Item,
    itemData,
    component: C = "div",
    header = null,
    footer = null,
    getKey = (i) => i,
    tabIndex = -1,
    style,
    ...rest
  } = props;
  const [outerRef, innerRef] = useSyncedStyles_default(model);
  return /* @__PURE__ */ jsxs(
    C,
    {
      ...rest,
      style: {
        overflow: "auto",
        contain: "strict",
        ...style
      },
      ref: useCallback(
        (el) => model.setScroller(el),
        [model]
      ),
      tabIndex,
      children: [
        header,
        /* @__PURE__ */ jsx("div", { ref: outerRef, children: /* @__PURE__ */ jsx("div", { ref: innerRef, children: /* @__PURE__ */ jsx(
          Subscription_default,
          {
            model,
            events: [VirtualScrollerEvent3.RANGE],
            children: () => mapVisibleRange(model, (i) => /* @__PURE__ */ jsx(
              Item,
              {
                model,
                i,
                data: itemData
              },
              getKey(i, itemData)
            ))
          }
        ) }) }),
        footer
      ]
    }
  );
};
var List_default = List;

// src/hooks/useScroller/index.ts
var useScroller = (model, scroller) => useIsomorphicLayoutEffect_default(() => {
  model.setScroller(scroller);
  return () => model.setScroller(null);
}, [model, scroller]);
var useScroller_default = useScroller;

// src/hooks/useSubscription/index.ts
import { useEffect as useEffect2 } from "react";
var useSubscription = (model, events, callBack) => useEffect2(() => {
  if (callBack) {
    callBack();
    return model.on(callBack, events);
  }
}, [model, callBack, events]);
var useSubscription_default = useSubscription;

// src/hooks/useVirtualModel/index.ts
import { useRef } from "react";
import {
  VirtualScroller as VirtualScroller3
} from "@af-utils/virtual-core";
var useVirtualModel = (params) => useRef().current ||= new VirtualScroller3(params);
var useVirtualModel_default = useVirtualModel;

// src/hooks/useVirtual/index.ts
var useVirtual = (params) => {
  const model = useVirtualModel_default(params);
  useIsomorphicLayoutEffect_default(() => model.set(params));
  return model;
};
var useVirtual_default = useVirtual;
export {
  List_default as List,
  Subscription_default as Subscription,
  mapVisibleRange,
  mapVisibleRangeWithOffset,
  useComponentSubscription_default as useComponentSubscription,
  useScroller_default as useScroller,
  useSubscription_default as useSubscription,
  useSyncedStyles_default as useSyncedStyles,
  useVirtual_default as useVirtual,
  useVirtualModel_default as useVirtualModel
};
